package info.textgrid.services.aggregator;

import info.textgrid.middleware.confclient.ConfservClient;
import info.textgrid.middleware.confclient.ConfservClientConstants;
import info.textgrid.middleware.tgsearch.client.SearchClient;
import info.textgrid.namespaces.metadata.core._2010.MetadataContainerType;
import info.textgrid.namespaces.metadata.core._2010.ObjectType;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.AuthFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.IoFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.MetadataParseFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.ObjectNotFoundFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.ProtocolNotImplementedFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.TGCrudService;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.TGCrudService_Service;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.activation.DataHandler;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;
import javax.xml.stream.XMLStreamException;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.MTOMFeature;

import org.codehaus.jettison.json.JSONException;

public class TextGridRepProvider implements ITextGridRep {

	private HashMap<String, String> config;
	private ConfservClient confservClient;
	private TGCrudService crud;
	private SearchClient publicSearchClient;
	private String CONF_ENDPOINT = "https://www.textgridlab.org/1.0/confserv";

	public String getCONF_ENDPOINT() {
		return CONF_ENDPOINT;
	}

	public void setCONF_ENDPOINT(final String cONF_ENDPOINT) {
		CONF_ENDPOINT = cONF_ENDPOINT;
	}

	private static final ITextGridRep instance = new TextGridRepProvider();

	@Override
	public String getConfValue(final String key) throws WebApplicationException {
		if (config == null) {
			try {
				confservClient = new ConfservClient(CONF_ENDPOINT);
				config = confservClient.getAll();
			} catch (final IOException e) {
				throw new WebApplicationException(e);
			} catch (final JSONException e) {
				throw new WebApplicationException(e);
			} catch (final XMLStreamException e) {
				throw new WebApplicationException(e);
			}
		}
		return config.get(key);
	}

	@Override
	public TGCrudService getCRUDService() {
		if (crud == null) {
			final URL wsdl = TGCrudService_Service.class
					.getResource("/wsdl/TGCrudService.wsdl");
			final TGCrudService_Service tgCrudService_Service = new TGCrudService_Service(
					wsdl);
			final TGCrudService _crud = tgCrudService_Service
					.getTGCrudPort(new MTOMFeature());
			final BindingProvider bp = (BindingProvider) _crud;
			final Map<String, Object> requestContext = bp.getRequestContext();
			requestContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
					getConfValue(ConfservClientConstants.TG_CRUD));
			crud = _crud;
		}
		return crud;
	}

	@Override
	public InputStream getContent(final URI uri, final String sid)
			throws ObjectNotFoundFault, MetadataParseFault, IoFault,
			ProtocolNotImplementedFault, AuthFault, IOException {
		final TGCrudService crudService = getCRUDService();
		final Holder<MetadataContainerType> mdHolder = new Holder<MetadataContainerType>();
		final Holder<DataHandler> dHolder = new Holder<DataHandler>();
		crudService.read(sid, null, uri.toString(), mdHolder, dHolder);
		return dHolder.value.getInputStream();
	}

	@Override
	public TGOSupplier<InputStream> read(final URI uri, final String sid)
			throws WebApplicationException {
		final TGCrudService crudService = getCRUDService();
		final Holder<MetadataContainerType> mdHolder = new Holder<MetadataContainerType>();
		final Holder<DataHandler> dHolder = new Holder<DataHandler>();
		try {
			crudService.read(sid, null, uri.toString(), mdHolder, dHolder);

			return new TGOSupplier<InputStream>() {

				@Override
				public InputStream getInput() throws IOException {
					return dHolder.value.getInputStream();
				}

				@Override
				public ObjectType getMetadata() {
					return mdHolder.value.getObject();
				}
			};
		} catch (final ObjectNotFoundFault e) {
			throw new WebApplicationException(e, Status.NOT_FOUND);
		} catch (final MetadataParseFault e) {
			throw new WebApplicationException(e);
		} catch (final IoFault e) {
			throw new WebApplicationException(e);
		} catch (final ProtocolNotImplementedFault e) {
			throw new WebApplicationException(e, Status.BAD_REQUEST);
		} catch (final AuthFault e) {
			throw new WebApplicationException(e, Status.FORBIDDEN);
		}
	}

	@Override
	public InputStream getContent(final URI uri) throws ObjectNotFoundFault,
			MetadataParseFault, IoFault, ProtocolNotImplementedFault,
			AuthFault, IOException {
		return getContent(uri, null);
	}

	@Override
	public SearchClient getPublicSearchClient() {
		if (publicSearchClient == null) {
			publicSearchClient = new SearchClient(
					getConfValue(ConfservClientConstants.TG_SEARCH_PUBLIC));
		}
		return publicSearchClient;
	}

	/**
	 * Creates a search client for the non-public repository, readily
	 * initialized with the given session id. Note that this must be cached by
	 * the client.
	 * 
	 * @param sid
	 * @return
	 */
	@Override
	public SearchClient createPrivateSearchClient(final String sid) {
		final SearchClient client = new SearchClient(
				getConfValue(ConfservClientConstants.TG_SEARCH));
		client.setSid(sid);
		return client;
	}

	public static ITextGridRep getInstance() {
		return instance;
	}

	@Override
	public String getCRUDRestEndpoint() {
		final String soapEndpoint = getConfValue(ConfservClientConstants.TG_CRUD);
		return soapEndpoint.substring(0, soapEndpoint.lastIndexOf('/') + 1)
				.concat("rest");
	}

}
