package info.textgrid.services.aggregator.tree;

import info.textgrid.namespaces.metadata.core._2010.ObjectType;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.AuthFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.IoFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.MetadataParseFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.ObjectNotFoundFault;
import info.textgrid.services.aggregator.AggregationTreeWalker;
import info.textgrid.services.aggregator.ITextGridRep;
import info.textgrid.utils.export.aggregations.Aggregation;
import info.textgrid.utils.export.aggregations.AggregationEntry;

import java.util.Deque;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.google.common.collect.Lists;
import com.ibm.icu.text.MessageFormat;

public class AggregationTreeFactory extends AggregationTreeWalker {

	private final Deque<Aggregation> stack = Lists.newLinkedList();
	private Aggregation root;
	private static Logger logger = Logger.getLogger("info.textgrid.utils.export.aggregations");

	protected AggregationTreeFactory(final ObjectType root, final ITextGridRep repository, final String sid) {
		super();
		this.setRepository(repository);
		this.setSid(sid);
//		this.root = new Aggregation(root);
//		stack.push(this.root);
		walkAggregation(root, false);
	}

	@Override
	protected boolean walk(final ObjectType object, final boolean again) {
		if (super.walk(object, again))
			return true; // Aggregation already handled via #walkAggregation
		else {
			new AggregationEntry(object, stack.peek());
			return true;
		}
	}

	@Override
	protected void walkAggregation(final ObjectType aggregation,
			final boolean again) {
		stack.push(new Aggregation(aggregation, stack.peek()));
		super.walkAggregation(aggregation, again);

		if (aggregation.getEdition() != null
				&& aggregation.getEdition().getIsEditionOf() != null) {
			final String workURI = aggregation.getEdition().getIsEditionOf();
			if (!seen.contains(workURI)) {
				try {
					final ObjectType workObject = getRepository()
							.getCRUDService()
							.readMetadata(getSid(), "", workURI).getObject();
					walkAggregation(workObject, again);
				} catch (final MetadataParseFault e) {
					logger.log(
							Level.WARNING,
							MessageFormat
									.format("Failed to retrieve work metadata for URI {0}, edition {1}",
											workURI, stack.peek()), e);
					walkInaccessibleURI(workURI);
				} catch (final ObjectNotFoundFault e) {
					logger.log(
							Level.WARNING,
							MessageFormat
									.format("Failed to retrieve work metadata for URI {0}, edition {1}",
											workURI, stack.peek()), e);
					walkInaccessibleURI(workURI);
				} catch (final IoFault e) {
					logger.log(
							Level.WARNING,
							MessageFormat
									.format("Failed to retrieve work metadata for URI {0}, edition {1}",
											workURI, stack.peek()), e);
					walkInaccessibleURI(workURI);
				} catch (final AuthFault e) {
					logger.log(
							Level.WARNING,
							MessageFormat
									.format("Failed to retrieve work metadata for URI {0}, edition {1}",
											workURI, stack.peek()), e);
					walkInaccessibleURI(workURI);
				}
			}
		}

		this.root = stack.pop();
	}

	protected Aggregation getRoot() {
		return root;
	}

	public static Aggregation create(final ObjectType root, final ITextGridRep repository, final String sid) {
		return new AggregationTreeFactory(root, repository, sid).getRoot();
	}

}
