package info.textgrid.services.aggregator.zip;

import info.textgrid.namespaces.metadata.core._2010.MetadataContainerType;
import info.textgrid.namespaces.metadata.core._2010.ObjectType;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.AuthFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.IoFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.MetadataParseFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.ObjectNotFoundFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.TGCrudService;
import info.textgrid.services.aggregator.ITextGridRep;
import info.textgrid.services.aggregator.RESTUtils;
import info.textgrid.utils.export.filenames.DefaultFilenamePolicy;

import java.net.URI;
import java.text.MessageFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.ServletContext;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.apache.cxf.jaxrs.model.wadl.Description;

@Path("/zip")
public class ZIP {
	private final ITextGridRep repository;
	final Logger logger = Logger.getLogger(ZIP.class.getCanonicalName());

	@Context
	private ServletContext servlet;
	
	public ZIP(final ITextGridRep repository) {
		this.repository = repository;
	}
	
	@GET
	@Path(value = "/{object}")
	@Produces(value = "application/zip")
	public Response get(
			@Description("The TextGrid URI of the TEI document or aggregation to transform")
			@PathParam("object") final URI uri,
			@Description("Session ID to access protected resources")
			@QueryParam("sid") final String sid) throws MetadataParseFault, ObjectNotFoundFault, IoFault, AuthFault {
		
		final TGCrudService crud = repository.getCRUDService();
		final MetadataContainerType container = crud.readMetadata(sid, null, uri.toString());
		final ObjectType rootObject = container.getObject();
		
		final String format = rootObject.getGeneric().getProvided().getFormat();
		if (!format.contains("aggregation")) {
			logger.log(Level.SEVERE, "The requested object {0} is a {1}, not an aggregation, and thus cannot be aggregated.", new Object[] {uri,format});
			throw new WebApplicationException(javax.ws.rs.core.Response
					.status(Status.BAD_REQUEST)
					.entity(MessageFormat.format(
							"The requested object {0} is a {1}, not an aggregation, and thus cannot be aggregated.", uri,
							format)).build());
		}
		
		return RESTUtils
				.attachmentResponse(DefaultFilenamePolicy.INSTANCE.translate(rootObject.getGeneric().getProvided().getTitle().get(0)).concat(".zip"))
				.type("application/zip")
				.entity(new ZipResult(rootObject, repository, sid))
.build();
	}

//	private void appendLS(final Aggregation aggregation, final StringBuilder output) {
//		output.append(aggregation.getFileName(true)).append(":\n");
//		for(final IAggregationEntry child : aggregation.getChildren())
//			if (child instanceof Aggregation) {
//				appendLS((Aggregation) child, output);
//			} else {
//				output.append(' ').append(child.getFileName(true)).append('\n');
//			}
//	}

}
