package info.textgrid.services.aggregator.teicorpus;

import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.AuthFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.IoFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.MetadataParseFault;
import info.textgrid.namespaces.middleware.tgcrud.services.tgcrudservice.ObjectNotFoundFault;
import info.textgrid.services.aggregator.TextGridRepProvider;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;

import org.apache.cxf.helpers.IOUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class TEICorpusTest {

	private TEICorpus teiCorpus;

	@Before
	public void setUp() throws Exception {
		teiCorpus = new TEICorpus(TextGridRepProvider.getInstance());
	}

	@Test(expected = ObjectNotFoundFault.class)
	public void testGet404() throws URISyntaxException, ObjectNotFoundFault, MetadataParseFault, IoFault, AuthFault {
		final Response response = teiCorpus.get(URI.create("textgrid:doesnotexist"),
				true, false, null);
		Assert.assertEquals(404, response.getStatus());
	}

	@Test
	public void testGet() throws URISyntaxException, WebApplicationException,
	IOException, ObjectNotFoundFault, MetadataParseFault, IoFault, AuthFault {
		final Response response = teiCorpus.get(URI.create("textgrid:jmzg.0"), true,
				false, null);
		final Object entity = response.getEntity();
		final ByteArrayOutputStream output = new ByteArrayOutputStream();
		((StreamingOutput) entity).write(output);
		final InputStream inputStream = getClass().getClassLoader()
				.getResourceAsStream("gedichte-anakreons-corpus.xml");
		final String expected = IOUtils.toString(inputStream, "UTF-8");
		Assert.assertEquals(expected, output.toString("UTF-8"));
	}

}
